from orangewidget.widget import OWBaseWidget, Output, Input
from orangewidget.settings import Setting
from orangewidget import gui
from orangewidget.utils.widgetpreview import WidgetPreview
import Orange.data
from Orange.data.pandas_compat import table_from_frame, table_to_frame
import requests 
import pandas as pd
from datetime import datetime

def get_mobile_track(name):
    try:
        df = pd.DataFrame(requests.get(f"http://api.canair.io:8080/tracks/{name}").json()['data'])
        df['station'] = name
        df['station'] = pd.Categorical(df['station'])
        df['timestamp'] = df.timestamp.apply(lambda x: datetime.fromtimestamp(x))
    except:
        print(f"Fallo: {name}")
        df = pd.DataFrame()
    return df

class ExtraInfoWidget(OWBaseWidget):
    # Widget's name as displayed in the canvas
    name = "Track - Mobile Station"
    # Short widget description
    description = "Get extra information from mobile station tracks through CanAIRio API"

    # An icon resource file path for this widget
    # (a path relative to the module where this widget is defined)
    icon = "icons/circle-info-solid-rosa.png"
    priority = 9

    # Basic (convenience) GUI definition:
    #   a simple 'single column' GUI layout
    want_main_area = False
    #   with a fixed non resizable geometry.
    resizing_enabled = False
    
    # We want the current number entered by the user to be saved and restored when saving/loading a workflow. 
    # We can achieve this by declaring a special property/member in the widget’s class definition like so: 
    #id_obs = Setting(0, schema_only=True)
    obs_table = Setting("", schema_only=True)

    # Widget's outputs; here, a single output named "Number", of type int

    class Inputs:
        data = Input("Data", Orange.data.Table, auto_summary=False)

    class Outputs:
        extra_data = Output("extra", Orange.data.Table, auto_summary=False)

    want_main_area = False

    def __init__(self):
        # use the init method from the class OWBaseWidget
        super().__init__()
        self.dataset = None

        # info area
        info = gui.widgetBox(self.controlArea, "Info")

        self.infoa = gui.widgetLabel(info, 'No observations fetched yet.')
        self.infob = gui.widgetLabel(info, '')

        gui.separator(self.controlArea)

        # commit area
        #self.commitBox = gui.widgetBox(self.controlArea, "", spacing=2)
        #gui.button(self.commitBox, self, "Commit", callback=self.commit)
        #self.searchBox.setDisabled(True)

    def info_searching(self):
        self.infoa.setText('Searching...')

    @Inputs.data
    def set_data(self, dataset):
        if dataset is not None:
            self.dataset = dataset
            self.infoa.setText('%d instances in input dataset' % len(dataset))
            self.infob.setText("")
            
        else:
            self.dataset = None           
            self.infoa.setText('No data on input yet, waiting to get something.')
            self.infob.setText('')

        self.commit()
    
    def commit(self):
        if self.dataset is not None:
            df = table_to_frame(self.dataset)
            
            ids = df['name'].to_list()
            progress = gui.ProgressBar(self, len(df))

            observations = pd.DataFrame()
            for id_num in ids:
                obs = get_mobile_track(id_num)
                observations = pd.concat([observations, obs])
                progress.advance()
            progress.finish()

            self.obs_table = table_from_frame(observations)

            self.infoa.setText(f'{len(self.dataset)} instances in input dataset')
            self.infob.setText(f"{len(self.obs_table)} instances in output dataset")
            self.info.set_output_summary(len(observations))
        else:
            self.infoa.setText('No data on input yet, waiting to get something.')
            self.infob.setText('')
            self.info.set_output_summary(self.info.NoOutput)

        self.Outputs.extra_data.send(self.obs_table)


if __name__ == "__main__":
    WidgetPreview(ExtraInfoWidget).run()



from orangewidget.widget import OWBaseWidget, Output
from orangewidget.settings import Setting
from orangewidget import gui
from orangewidget.utils.widgetpreview import WidgetPreview
import Orange.data
from Orange.data.pandas_compat import table_from_frame
from mecoda_nat import get_obs, get_dfs
from typing import Optional, List
import pandas as pd

class NatusferaWidget(OWBaseWidget):
    
    # Widget's name as displayed in the canvas
    name = "Natusfera Obs."
    # Short widget description
    description = "Get observations from the Natusfera API"

    # An icon resource file path for this widget
    # (a path relative to the module where this widget is defined)
    #icon = "icons/N2.png"
    icon = "icons/natusfera_v1.png"
    priority = 30

    # Basic (convenience) GUI definition:
    #   a simple 'single column' GUI layout
    want_main_area = False
    #   with a fixed non resizable geometry.
    resizing_enabled = True

    # We want the current number entered by the user to be saved and restored when saving/loading a workflow. 
    # We can achieve this by declaring a special property/member in the widget’s class definition like so: 
    id_obs = Setting("", schema_only=True)
    project_name = Setting("", schema_only=True)
    query = Setting("", schema_only=True)
    user = Setting("", schema_only=True)
    taxon = Setting("", schema_only=True)
    place_name = Setting("", schema_only=True)
    year = Setting("", schema_only=True)
    num_max = Setting(20000, schema_only=True)

    # Widget's outputs; here, a single output named "Number", of type int
    class Outputs:
        observations = Output("Observations", Orange.data.Table, auto_summary=False)
        photos = Output("Photos", Orange.data.Table, auto_summary=False)

    def __init__(self):
        # llama el método init de la clase padre OWBaseWidget
        super().__init__()

        from AnyQt.QtGui import QIntValidator
        
        box = gui.widgetBox(self.controlArea, "Info")
        self.infoa = gui.widgetLabel(box, 'No observations fetched yet.')
        self.infob = gui.widgetLabel(box, '')

        gui.separator(self.controlArea)
        
        self.searchBox = gui.widgetBox(self.controlArea, "Search fields")

        #gui.spin(self.searchBox, self, 'id_obs',
        #         minv=0, maxv=400000, step=1, label='Enter an id of observation:')
        self.query_line = gui.lineEdit(self.searchBox, self, "query", label="Search by words:", orientation=1, controlWidth=140)
        self.project_line = gui.lineEdit(self.searchBox, self, 'project_name', label="Project name:", orientation=1, callback=self.project_name_edit, controlWidth=140)
        self.user_line = gui.lineEdit(self.searchBox, self, "user", label="User name:", orientation=1, controlWidth=140, callback=self.user_edit)
        self.place_line = gui.lineEdit(self.searchBox, self, "place_name", label="Place:", orientation=1, controlWidth=140)
        self.taxon_line = gui.comboBox(
            self.searchBox, 
            self, 
            "taxon", 
            box=None, 
            label="Taxon:", 
            labelWidth=None, 
            items=('', 'Animalia', 'Actinopterygii', 'Aves', 'Reptilia', 'Amphibia','Mammalia', 'Arachnida', 'Insecta', 'Plantae', 'Fungi', 'Protozoa', 'Mollusca', 'Chromista'), callback=None, 
            sendSelectedValue=True, 
            emptyString=False, 
            editable=False, 
            contentsLength=None, 
            searchable=False, 
            orientation=1,
            )
        self.year_line = gui.lineEdit(
            self.searchBox, 
            self, 
            "year", 
            label="Year:", 
            orientation=1, 
            controlWidth=80, 
            valueType=int, 
            validator=QIntValidator()
            )
        self.id_obs_line = gui.lineEdit(self.searchBox, self, 'id_obs', label="Id of observation:", callback=self.id_obs_edit, orientation=1, controlWidth=80)
        
        #self.project_line.setDisabled(True)
        
        #gui.hSlider(self.searchBox, self, "year", box=None, minValue=2000, maxValue=2021, step=1, label="Year", labelFormat=' %d', ticks=False, divideFactor=1.0, vertical=False, createLabel=True)
        gui.separator(self.controlArea)

        self.maxBox = gui.widgetBox(self.controlArea, "", spacing=1)
        #self.max_line = gui.lineEdit(self.maxBox, self, "num_max", label="", orientation=1)
        self.max_line = gui.spin(
            self.maxBox, 
            self,
            "num_max",
            minv=0, 
            maxv=20000, 
            step=200, 
            label='Max number of results:', 
            orientation=1)

        self.commitBox = gui.widgetBox(self.controlArea, "", spacing=2)
        gui.button(self.commitBox, self, "Commit", callback=self.commit)
        
    def info_searching(self):
        self.infoa.setText('Searching...')

    def id_obs_edit(self):
        if self.id_obs != "":
            self.project_line.setDisabled(True)
            self.query_line.setDisabled(True)
            self.user_line.setDisabled(True)
            self.taxon_line.setDisabled(True)
            self.place_line.setDisabled(True)
            self.year_line.setDisabled(True)
            #self.max_line.setDisabled(True)
            self.project_name = ""
            self.query = ""
            self.user = ""
            self.taxon = ""
            self.place_name = ""
            self.year = ""
            self.num_max = ""
        else:
            self.project_line.setDisabled(False)
            self.query_line.setDisabled(False)
            self.user_line.setDisabled(False)
            self.taxon_line.setDisabled(False)
            self.place_line.setDisabled(False)
            self.year_line.setDisabled(False)
            self.max_line.setDisabled(False) 

    def project_name_edit(self):
        if self.project_name != "":
            self.user_line.setDisabled(True)
            self.id_obs_line.setDisabled(True)
            self.taxon_line.setDisabled(True)
            self.user = ""
            self.id_obs = ""
            self.taxon = ""
        else:
            self.user_line.setDisabled(False)
            self.id_obs_line.setDisabled(False)
            self.taxon_line.setDisabled(False)

    def user_edit(self):
        if self.user != "":
            self.id_obs_line.setDisabled(True)
            self.project_line.setDisabled(True)
            self.id_obs = ""
            self.project_name = ""
        else:
            self.id_obs_line.setDisabled(False)
            self.project_line.setDisabled(False)


    def commit(self):
        self.infoa.setText(f'Searching...')
        self.infob.setText(f'')
        try:
            if self.id_obs == "":
                id_obs = None
            else:
                id_obs = int(self.id_obs)
            
            if self.project_name == "":
                project_name = None    
            else:
                project_name = self.project_name

            if self.taxon == "":
                taxon = None
            else:
                taxon = self.taxon

            if self.year == "":
                year = None
            else:
                year = int(self.year)

            if self.query == "":
                query = None
            else:
                query = self.query

            if self.user == "":
                user = None
            else:
                user = self.user

            if self.place_name == "":
                place_name = None
            else:
                place_name = self.place_name

            progress = gui.ProgressBar(self, 2)
            progress.advance()
            
            observations = get_obs(
                id_obs=id_obs, 
                project_name=project_name,
                query=query,
                user=user,
                taxon=taxon,
                place_name=place_name,
                year=year,
                num_max=self.num_max,
                )

            if len(observations) > 0:
                self.df_obs, self.df_photos = get_dfs(observations)
                self.df_obs['taxon_name'] = self.df_obs['taxon_name'].str.lower()
                self.df_photos['taxon_name'] = self.df_photos['taxon_name'].str.lower()

                self.infoa.setText(f'{len(observations)} observations gathered')
                self.infob.setText(f'{len(self.df_photos)} photos gathered')
                
                self.Outputs.observations.send(table_from_frame(self.df_obs))

                table_photos = table_from_frame(self.df_photos)
                for meta in table_photos.domain.metas:
                    if meta.name == "photos.medium_url":
                        meta.attributes = {"type": "image"}

                self.Outputs.photos.send(table_photos)
                self.info.set_output_summary(len(observations))

            else:
                self.infoa.setText(f'Nothing found.')
                self.infob.setText("")
                self.info.set_output_summary(self.info.NoOutput)

        except ValueError:
            self.infoa.setText(f'Nothing found.')
            self.infob.setText("")
        except Exception as error:
            self.infoa.setText(f'ERROR: \n{str(error)}')
            self.infob.setText("")
            
        progress.finish()    
        
if __name__ == "__main__":
    WidgetPreview(NatusferaWidget).run()




from orangewidget.widget import OWBaseWidget, Output, Input
from orangewidget.settings import Setting
from orangewidget import gui
from orangewidget.utils.widgetpreview import WidgetPreview
import Orange.data
from Orange.data.pandas_compat import table_from_frame, table_to_frame
import requests 
import pandas as pd
import flat_table
from datetime import datetime

def get_historic_data_fixed_station(st) -> pd.DataFrame:
    url = f"http://api.canair.io:8080/dwc/stations/{st}"
    data = requests.get(url).json()
    data = pd.DataFrame(data)
    obs = flat_table.normalize(data)
    obs.rename(columns=lambda s: s.replace("measurements.", ""), inplace=True)
    obs.rename(columns={'measurementDeterminedDate ': 'measurementDeterminedDate'}, inplace=True)
    cols = [
        'measurementDeterminedDate', 
        'observedOn', 'measurementID']
    obs[cols] = obs[cols].apply(pd.to_datetime)
    #obs = obs.drop(['index'], axis=1)
    return obs

class ExtraInfoWidget(OWBaseWidget):
    # Widget's name as displayed in the canvas
    name = "Last Hour Fixed Station"
    # Short widget description
    description = "Get extra information from fixed stations through CanAIRio API"

    # An icon resource file path for this widget
    # (a path relative to the module where this widget is defined)
    icon = "icons/circle-info-solid-gris.png"
    priority = 7

    # Basic (convenience) GUI definition:
    #   a simple 'single column' GUI layout
    want_main_area = False
    #   with a fixed non resizable geometry.
    resizing_enabled = False
    
    # We want the current number entered by the user to be saved and restored when saving/loading a workflow. 
    # We can achieve this by declaring a special property/member in the widget’s class definition like so: 
    obs_table = Setting("", schema_only=True)

    # Widget's outputs; here, a single output named "Number", of type int

    class Inputs:
        data = Input("Data", Orange.data.Table, auto_summary=False)

    class Outputs:
        extra_data = Output("extra", Orange.data.Table, auto_summary=False)

    want_main_area = False

    def __init__(self):
        # use the init method from the class OWBaseWidget
        super().__init__()
        self.dataset = None

        # info area
        info = gui.widgetBox(self.controlArea, "Info")

        self.infoa = gui.widgetLabel(info, 'No observations fetched yet.')
        self.infob = gui.widgetLabel(info, '')

        gui.separator(self.controlArea)

        # searchBox area
        #self.searchBox = gui.widgetBox(self.controlArea, "Search fields")

        # commit area
        #self.commitBox = gui.widgetBox(self.controlArea, "", spacing=2)
        #gui.button(self.commitBox, self, "Commit", callback=self.commit)
        #self.searchBox.setDisabled(True)

    def info_searching(self):
        self.infoa.setText('Searching...')

    @Inputs.data
    def set_data(self, dataset):
        if dataset is not None:
            self.dataset = dataset
            self.infoa.setText('%d instances in input dataset' % len(dataset))
            
        else:
            self.dataset = None           
            self.infoa.setText('No data on input yet, waiting to get something.')
            self.infob.setText('')
        self.commit()
    
    def selection(self):
        if self.dataset is None:
            return
        df = table_to_frame(self.dataset)
        ids = df['station_name'].to_list()
        progress = gui.ProgressBar(self, len(df))
        
        observations = pd.DataFrame()
        for id_num in ids:
            obs = get_historic_data_fixed_station(id_num)
            observations = pd.concat([observations, obs])
            progress.advance()

        progress.finish()     

        if len(df['measurementType'].unique()) == 1:
            observations = observations[observations['measurementType'] == df['measurementType'].unique()]

        self.obs_table = table_from_frame(observations)

    def commit(self):
        if self.dataset is not None:

            df = table_to_frame(self.dataset)

            ids = df['station_name'].to_list()
            progress = gui.ProgressBar(self, len(df))
            observations = pd.DataFrame()
            for id_num in ids:
                obs = get_historic_data_fixed_station(id_num)
                observations = pd.concat([observations, obs])
                progress.advance()
            progress.finish()

            types = df['measurementType'].unique()
            for t in types:
                observations = observations[observations['measurementType'] == t]
            
            observations.measurementValue = observations.measurementValue.astype(float)
            self.obs_table = table_from_frame(observations)

            

            self.infoa.setText(f'{len(self.dataset)} instances in input dataset')
            self.infob.setText(f"{len(self.obs_table)} instances in output dataset")
            self.info.set_output_summary(len(observations))

        else:
            self.infoa.setText('No data on input yet, waiting to get something.')
            self.infob.setText('')
            self.info.set_output_summary(self.info.NoOutput)

        self.Outputs.extra_data.send(self.obs_table)


if __name__ == "__main__":
    WidgetPreview(ExtraInfoWidget).run()



from Orange.data import Table
from Orange.widgets import gui
from Orange.widgets.settings import Setting
from Orange.widgets.widget import OWWidget, Input, Output, Msg
from Orange.widgets.utils.widgetpreview import WidgetPreview 

import sys
from PyQt5.QtWidgets import QApplication, QWidget, QInputDialog, QLineEdit, QFileDialog
from PyQt5.QtGui import QIcon

class MyWidget(QWidget):

    def __init__(self):
        super().__init__()

        self.title = 'PyQt5 file dialogs - pythonspot.com'
        self.left = 10
        self.top = 10
        self.width = 640
        self.height = 480


    def initUI(self):
        self.setWindowTitle(self.title)
        self.setGeometry(self.left, self.top, self.width, self.height)
        
        self.openFileNameDialog()
        self.openFileNamesDialog()
        self.saveFileDialog()
        
        self.show()
    
    def openFileNameDialog(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        fileName, _ = QFileDialog.getOpenFileName(self,"QFileDialog.getOpenFileName()", "","All Files (*);;Python Files (*.py)", options=options)
        if fileName:
            print(fileName)
    
    def openFileNamesDialog(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        files, _ = QFileDialog.getOpenFileNames(self,"QFileDialog.getOpenFileNames()", "","All Files (*);;Python Files (*.py)", options=options)
        if files:
            print(files)
    
    def saveFileDialog(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        fileName, _ = QFileDialog.getSaveFileName(self,"QFileDialog.getSaveFileName()","","All Files (*);;Text Files (*.txt)", options=options)
        if fileName:
            print(fileName)


if __name__ == "__main__":
    WidgetPreview(MyWidget).run()

    app = QApplication(sys.argv)
    #ex = App()
    sys.exit(app.exec_())
    
